@@ -887,7 +887,7 @@ class Pile(Widget): # either FlowWidget or BoxWidget
     
     def get_cursor_coords(self, size):
         """Return the cursor coordinates of the focus widget."""
-        if not self.focus_item.selectable():
+        if not self.focus_item or not self.focus_item.selectable():
             return None
         if not hasattr(self.focus_item,'get_cursor_coords'):
             return None
@@ -926,6 +926,8 @@ class Pile(Widget): # either FlowWidget or BoxWidget
     def keypress(self, size, key ):
         """Pass the keypress to the widget in focus.
         Unhandled 'up' and 'down' keys may cause a focus change."""
+        if not self.focus_item:
+            return key
 
         item_rows = None
         if len(size)==2:
@@ -998,6 +1000,8 @@ class Pile(Widget): # either FlowWidget or BoxWidget
             if wrow+r > row:
                 break
             wrow += r
+        else:
+            return False
 
         if not w.selectable():
             return False
