@@ -21,10 +21,6 @@
 # Urwid web site: http://excess.org/urwid/
 
 import unittest
-try:
-    from test import test_support
-except ImportError:
-    from test import support as test_support
 from doctest import DocTestSuite, ELLIPSIS, IGNORE_EXCEPTION_DETAIL
 
 import urwid
@@ -846,9 +842,10 @@ class TextTest(unittest.TestCase):
 
 class EditTest(unittest.TestCase):
     def setUp(self):
-        self.t1 = urwid.Edit("","blah blah")
-        self.t2 = urwid.Edit("stuff:", "blah blah")
-        self.t3 = urwid.Edit("junk:\n","blah blah\n\nbloo",1)
+        self.t1 = urwid.Edit(B(""),"blah blah")
+        self.t2 = urwid.Edit(B("stuff:"), "blah blah")
+        self.t3 = urwid.Edit(B("junk:\n"),"blah blah\n\nbloo",1)
+        self.t4 = urwid.Edit(u"better:")
 
     def ktest(self, e, key, expected, pos, desc):
         got= e.keypress((12,),key)
@@ -893,6 +890,15 @@ class EditTest(unittest.TestCase):
         self.ktest(self.t3,'down',None,15,"down line 2 to 3")
         self.ktest(self.t3,'down','down',15,"down at bottom")
 
+    def test_utf8_input(self):
+        urwid.set_encoding("utf-8")
+        self.t1.set_edit_text('')
+        self.t1.keypress((12,), u'û')
+        self.assertEquals(self.t1.edit_text, u'û'.encode('utf-8'))
+        self.t4.keypress((12,), u'û')
+        self.assertEquals(self.t4.edit_text, u'û')
+
+
 class EditRenderTest(unittest.TestCase):
     def rtest(self, w, expected_text, expected_cursor):
         expected_text = [B(t) for t in expected_text]
@@ -1171,7 +1177,7 @@ class ListBoxRenderTest(unittest.TestCase):
         assert cursor == exp_cur, "%s (cursor) got: %r expected: %r" %(desc,cursor,exp_cur)
 
 
-    def test1Simple(self):
+    def test1_simple(self):
         T = urwid.Text
 
         self.ltest( "simple one text item render",
@@ -1186,7 +1192,7 @@ class ListBoxRenderTest(unittest.TestCase):
             [T("1"),T("2"),T("3\n4"),T("5"),T("6")], 2, 1,
             ["2   ","3   ","4   ","5   ","6   "],None)
 
-    def test2Trim(self):
+    def test2_trim(self):
         T = urwid.Text
 
         self.ltest( "trim unfocused bottom",
@@ -1213,7 +1219,7 @@ class ListBoxRenderTest(unittest.TestCase):
             [T("1\n2\n3\n4\n5\n6\n7")], 0, -1,
             ["2   ","3   ","4   ","5   ","6   "],None)
 
-    def test3Shift(self):
+    def test3_shift(self):
         T,E = urwid.Text, urwid.Edit
 
         self.ltest( "shift up one fit",
@@ -1240,6 +1246,15 @@ class ListBoxRenderTest(unittest.TestCase):
             [T("1\n2"),T("3"),T("4"),e], 3, 4,
             ["1   ","2   ","3   ","4   ","abc "], (3,4))
 
+    def test4_really_large_contents(self):
+        T,E = urwid.Text, urwid.Edit
+        self.ltest("really large edit",
+            [T(u"hello"*100)], 0, 0,
+            ["hell","ohel","lohe","lloh","ello"], None)
+
+        self.ltest("really large edit",
+            [E(u"", u"hello"*100)], 0, 0,
+            ["hell","ohel","lohe","lloh","llo "], (3,4))
 
 class ListBoxKeypressTest(unittest.TestCase):
 
@@ -1702,6 +1717,48 @@ class ListBoxKeypressTest(unittest.TestCase):
             4,3, None )
 
 
+class ZeroHeightContentsTest(unittest.TestCase):
+    def test_listbox_pile(self):
+        lb = urwid.ListBox(urwid.SimpleListWalker(
+            [urwid.Pile([])]))
+        lb.render((40,10), focus=True)
+
+    def test_listbux_text_pile_page_down(self):
+        lb = urwid.ListBox(urwid.SimpleListWalker(
+            [urwid.Text(u'above'), urwid.Pile([])]))
+        lb.keypress((40,10), 'page down')
+        self.assertEquals(lb.get_focus()[1], 0)
+        lb.keypress((40,10), 'page down') # second one caused ListBox failure
+        self.assertEquals(lb.get_focus()[1], 0)
+
+    def test_listbux_text_pile_page_up(self):
+        lb = urwid.ListBox(urwid.SimpleListWalker(
+            [urwid.Pile([]), urwid.Text(u'below')]))
+        lb.set_focus(1)
+        lb.keypress((40,10), 'page up')
+        self.assertEquals(lb.get_focus()[1], 1)
+        lb.keypress((40,10), 'page up') # second one caused pile failure
+        self.assertEquals(lb.get_focus()[1], 1)
+
+    def test_listbux_text_pile_down(self):
+        sp = urwid.Pile([])
+        sp.selectable = lambda: True # abuse our Pile
+        lb = urwid.ListBox(urwid.SimpleListWalker([urwid.Text(u'above'), sp]))
+        lb.keypress((40,10), 'down')
+        self.assertEquals(lb.get_focus()[1], 0)
+        lb.keypress((40,10), 'down')
+        self.assertEquals(lb.get_focus()[1], 0)
+
+    def test_listbux_text_pile_up(self):
+        sp = urwid.Pile([])
+        sp.selectable = lambda: True # abuse our Pile
+        lb = urwid.ListBox(urwid.SimpleListWalker([sp, urwid.Text(u'below')]))
+        lb.set_focus(1)
+        lb.keypress((40,10), 'up')
+        self.assertEquals(lb.get_focus()[1], 1)
+        lb.keypress((40,10), 'up')
+        self.assertEquals(lb.get_focus()[1], 1)
+
 class PaddingTest(unittest.TestCase):
     def ptest(self, desc, align, width, maxcol, left, right,min_width=None):
         p = urwid.Padding(None, align, width, min_width)
@@ -1773,6 +1830,25 @@ class PaddingTest(unittest.TestCase):
         self.mctest("cursor right edge",2,2,(10,2),7,5)
         self.mctest("cursor right edge+1",2,2,(10,2),8,5)
 
+    def test_reduced_padding_cursor(self):
+        # FIXME: This is at least consistent now, but I don't like it.
+        # pack() on an Edit should leave room for the cursor
+        # fixing this gets deep into things like Edit._shift_view_to_cursor
+        # though, so this might not get fixed for a while
+
+        p = urwid.Padding(urwid.Edit(u'',u''), width='pack', left=4)
+        self.assertEquals(p.render((10,), True).cursor, None)
+        self.assertEquals(p.get_cursor_coords((10,)), None)
+        self.assertEquals(p.render((4,), True).cursor, None)
+        self.assertEquals(p.get_cursor_coords((4,)), None)
+
+        p = urwid.Padding(urwid.Edit(u'',u''), width=('relative', 100), left=4)
+        self.assertEquals(p.render((10,), True).cursor, (4, 0))
+        self.assertEquals(p.get_cursor_coords((10,)), (4, 0))
+        self.assertEquals(p.render((4,), True).cursor, None)
+        self.assertEquals(p.get_cursor_coords((4,)), None)
+
+
 
 
 class FillerTest(unittest.TestCase):
@@ -2031,7 +2107,7 @@ class LineBoxTest(unittest.TestCase):
                 B("\xe2\x94\x90"), B("\xe2\x94\x82"), B("\xe2\x94\x82"),
                 B("\xe2\x94\x94"), B("\xe2\x94\x80"), B("\xe2\x94\x98")))
 
-        nums = [B(str(n)) for n in range(8)] 
+        nums = [B(str(n)) for n in range(8)]
         b = dict(zip(["tlcorner", "tline", "trcorner", "lline", "rline",
             "blcorner", "bline", "brcorner"], nums))
         l = urwid.LineBox(t, **b).render((3,)).text
@@ -2329,6 +2405,7 @@ def test_all():
         ListBoxChangeFocusTest,
         ListBoxRenderTest,
         ListBoxKeypressTest,
+        ZeroHeightContentsTest,
         PaddingTest,
         FillerTest,
         FrameTest,
@@ -2363,4 +2440,4 @@ def test_all():
     return tests
 
 if __name__ == '__main__':
-    test_support.run_unittest(test_all())
+    unittest.TextTestRunner(verbosity=2).run(test_all())
